﻿;******************************************************************************
;	電源プラン関連ツール共通関数ライブラリ
;	20231230
;
;	By NightWotk
;******************************************************************************
Return


;******************************************************************************
; 電源プランに関する関数
;******************************************************************************
;------------------------------------------------------------------------------
; アクティブ電源プランGUID取得
;------------------------------------------------------------------------------
;	戻り値		アクティブプランのGUID
;------------------------------------------------------------------------------
PS_GetActiveGuid(){
	;変数初期化
	guid	:= 0

	;レジストリから現在のGUID取得
	RegRead	, guid
		, HKEY_LOCAL_MACHINE
		, SYSTEM\CurrentControlSet\Control\Power\User\PowerSchemes
		, ActivePowerScheme

	Return guid
}


;------------------------------------------------------------------------------
; 電源プラン一覧取得
;------------------------------------------------------------------------------
;	戻り値		電源プランの一覧
;------------------------------------------------------------------------------
PS_GetList(){
	;定数と変数初期化
	BATCH_FILE		:= "Core\GetPowerSchemeList.bat"	;実行するバッチファイル名
	SCHEMELIST_FILE		:= "Core\GetPowerSchemeListResult.log"	;電源プラン一覧ファイル
	list		:= ""

	;電源プラン一覧書き出し、実行失敗時は終了
	RunWait, %BATCH_FILE%,, Hide UseErrorLevel
	If ( ErrorLevel == "ERROR")
	{
		MsgBox , 16, エラー, 「%BATCH_FILE%」実行エラー`nアプリケーションを終了します。
		ExitApp
	}

	;プランリスト読み込み、実行失敗時は終了
	FileRead, list, *t %SCHEMELIST_FILE%
	If ( ErrorLevel )
	{
		MsgBox , 16, エラー, 「%SCHEMELIST_FILE%」読込失敗`nアプリケーションを終了します。
		ExitApp
	}

	;プランリスト一時ファイル削除
	FileDelete	, %SCHEMELIST_FILE%

	Return list
}


;------------------------------------------------------------------------------
; GUIDから電源プラン名取得
;------------------------------------------------------------------------------
;	guid_target	取得したい電源プランのGUID
;	list		電源プランの一覧（＝PS_GetList()の結果）
;	戻り値		対象のプラン名、該当なし時は""
;------------------------------------------------------------------------------
PS_GetName(guid_target,list){
	;変数初期化
	name	:= ""
	guid	:= ""

	;プランリスト各行ループ
	Loop, Parse, list, `n
	{
		;GUID抽出
		guid := PS_FindGuid(A_LoopField)

		;GUIDが無い行は飛ばす
		If ( guid == "")
			Continue

		;取得対象？
		If ( guid == guid_target )
		{
			name	:= PS_FindLabel(A_LoopField)
			Break
		}
	}

	Return	name
}


;------------------------------------------------------------------------------
; 電源プラン名からGUID取得
;------------------------------------------------------------------------------
;	name_target	取得したい電源プランのGUID
;	list		電源プランの一覧（＝GetPowerSchemeList()の結果）
;	戻り値		対象のプラン名、該当なし時は""
;------------------------------------------------------------------------------
PS_GetGuid(name_target,list){
	;変数初期化
	name	:=
	guid	:= ""

	;プランリスト各行ループ
	Loop, Parse, list, `n
	{
		;GUID抽出
		name := PS_FindLabel(A_LoopField)

		;GUIDが無い行は飛ばす
		If ( name == "")
			Continue

		;取得対象？
		If ( name == name_target )
		{
			guid	:= PS_FindGuid(A_LoopField)
			Break
		}
	}

	Return	guid
}

;------------------------------------------------------------------------------
; GUIDをハイフン無表記に変換
;------------------------------------------------------------------------------
;	guid		GUID
;	戻り値		GUID（ハイフン無し）
;------------------------------------------------------------------------------
PS_GuidRemovalHyphen(guid=""){
	;引数未指定時はダミーデータを生成(ポカヨケ)
	If ( guid == "" )
		guid := "00000000-0000-0000-0000-000000000000"

	;ハイフンをすべて除去
	guid	:= FxStringReplace( guid, "-", "", 1)

	Return guid
}


;------------------------------------------------------------------------------
; GUIDをハイフン有表記に変換
;------------------------------------------------------------------------------
;	guid		GUID
;	戻り値		GUID（ハイフンあり）
;------------------------------------------------------------------------------
PS_GuidAddHyphen(guid=""){
	;引数未指定時はダミーデータを生成(ポカヨケ)
	If ( guid == "" )
		guid := "00000000-0000-0000-0000-000000000000"

	;一度ハイフン除去フィルターを通す
	guid	:= PS_GuidRemovalHyphen(guid)

	;ハイフン追加
	guid	:=	FxStringMid( guid, 0, 8)
		.	"-"
		.	FxStringMid( guid, 9, 4)
		.	"-"
		.	FxStringMid( guid, 13, 4)
		.	"-"
		.	FxStringMid( guid, 17, 4)
		.	"-"
		.	FxStringMid( guid, 21, 12)

	Return guid
}


;------------------------------------------------------------------------------
; 入力データからGUIDを検索して返す
;------------------------------------------------------------------------------
;	戻り値		最初に見つかったGUID、見つからなかった場合は""
;------------------------------------------------------------------------------
PS_FindGuid(text){
	;変数初期化
	guid	:= ""
	pos	:= 0

	;開始位置取得
	pos	:= RegExMatch( text, "[a-fA-F0-9]{8}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{4}-[a-fA-F0-9]{12}")

	;なければ終了
	If ( !pos )
		Return guid

	;GUID抽出
	guid	:= FxStringMid( text, pos, 36)

	Return guid
}


;------------------------------------------------------------------------------
; 入力データから項目名を検索して返す
;------------------------------------------------------------------------------
;	戻り値		最初に見つかった項目名、見つからなかった場合は""
;------------------------------------------------------------------------------
PS_FindLabel(text){
	;変数初期化
	label	:= ""
	pos	:= 0
	len	:= 0

	;開始位置と終了位置取得
	pos	:= RegExMatch( text, "\(.*\)" )
	len	:= FxStringGetPos( text, ")", "r")

	;なければ終了
	If ( !pos )
		Return label

	;項目名抽出
	label	:= FxStringMid( text, pos+1, len-pos)

	Return label
}


;------------------------------------------------------------------------------
; 入力データから設定値(0x00000000形式)を検索して返す
;------------------------------------------------------------------------------
;	戻り値		最初に見つかった設定値、見つからなかった場合は""
;------------------------------------------------------------------------------
PS_FindValue(text){
	;変数初期化
	data	:= ""
	pos	:= 0

	;開始位置取得
	pos	:= RegExMatch( text, "0x[0-9a-fA-F]{8}" )

	;なければ終了
	If ( !pos )
		Return data

	;項目名抽出
	data	:= FxStringMid( text, pos, 10)

	Return data
}
