﻿;******************************************************************************
;	電源プラン選択ツール
;
;	By NightWotk
;******************************************************************************

;******************************************************************************
; 起動と終了
;******************************************************************************
;------------------------------------------------------------------------------
; Autoexec
;------------------------------------------------------------------------------
	;環境設定
	#Persistent			;常駐
	#NoEnv				;環境変数無視
	#NoTrayIcon 			;トレイアイコン非表示起動
	#SingleInstance Force		;多重起動禁止
	SetWorkingDir,	%A_ScriptDir%	;作業ディレクトリ変更
	OnExit,	EndProcessing		;終了処理


	;定数用変数
	;ソフト情報設定
	APP_TILTE	:= "PowerSchemeSelecter"	;タイトル
	APP_VER		:= "20240103"			;バージョン


	;初期化
	TreyIcon_Init()					;トレイアイコンメニュー初期化


	;表示状態更新
	TreyIcon_Update()
	SetTimer, TreyIcon_Update_Timer, 1000		;自動更新タイマーセット
	SSTP_OnPowerSchemeUpDate(2)			;SSTP送信

	Menu, Tray, Icon				;トレイアイコン表示

	Return



;------------------------------------------------------------------------------
; 共通関数ライブラリ読込
;------------------------------------------------------------------------------
#Include Libs\functions.ahk
#Include Libs\functions_PS.ahk



;------------------------------------------------------------------------------
; 終了処理
;------------------------------------------------------------------------------
EndProcessing:
	ExitApp		;本終了
	Return



;******************************************************************************
; Guidテーブル関数
;******************************************************************************
;------------------------------------------------------------------------------
; Guidテーブル作成
;------------------------------------------------------------------------------
PowerSchemeArray_InitGuid(guidlist){
	global
	StringSplit, powerscheme_guidarray_, guidlist, `n
	Return powerscheme_guidarray_0
}

;------------------------------------------------------------------------------
; Guidテーブル項目取得
;------------------------------------------------------------------------------
PowerSchemeArray_GetGuid(index){
	global
	Return powerscheme_guidarray_%index%
}

;------------------------------------------------------------------------------
; Guidテーブルから適合する指定されたGuidのインデックス番号を返す
;------------------------------------------------------------------------------
PowerSchemeArray_GetPosGuid(guid){
	global
	Loop, %powerscheme_guidarray_0%
	{
		If (powerscheme_guidarray_%A_Index% == guid)
		{
			Return A_Index
		}
	}
	Return 0
}

;------------------------------------------------------------------------------
; プラン名テーブル作成
;------------------------------------------------------------------------------
PowerSchemeArray_InitName(namelist){
	global
	StringSplit, powerscheme_namearray_, namelist, `n
	Return powerscheme_namearray_0
}

;------------------------------------------------------------------------------
; プラン名テーブル項目取得
;------------------------------------------------------------------------------
PowerSchemeArray_GetName(index){
	global
	Return powerscheme_namearray_%index%
}



;******************************************************************************
; 各処理
;******************************************************************************
;------------------------------------------------------------------------------
; トレイアイコンメニュー 初期化
;------------------------------------------------------------------------------
TreyIcon_Init(){
	;変数初期化
	global				;【グローバル変数モード】
	local menu_name	:= ""		;メニュー項目処理用
	powerscheme_new_guid := 0	;変更フラグクリア

	GetPowerSchemeArrey()		;電源プラン一覧更新

	;初期設定
	Menu, Tray, DeleteAll					;メニュー内全項目を抹消
	Menu, Tray, Tip, %APP_TILTE%				;タイトル設定(仮)
	Menu, Tray, Icon, C:\Windows\System32\powercpl.dll, 3	;アイコン設定
	Menu, Tray, NoStandard					;標準項目抹消
	Menu, Tray, UseErrorLevel				;エラーダイアログOFF


	;電源プラン一覧
	Loop, %powerscheme_num%
	{
		menu_name := PowerSchemeArray_GetName(A_Index)
		Menu, Tray, Add, %menu_name%	, TrayMenu_ChangePowerScheme	;項目追加
	}

	;その他
	Menu, Tray, Add
	Menu, Tray, Add 	, ツール再起動	, TrayMenu_Reload		;再起動
	Menu, Tray, Add 	, ツール終了	, TrayMenu_End			;終了

	Return
}


;------------------------------------------------------------------------------
; トレイアイコンメニューからの操作
;------------------------------------------------------------------------------
;プラン切替
TrayMenu_ChangePowerScheme:
	;電源プラン変更
	powerscheme_new_guid	:=  PowerSchemeArray_GetGuid(A_ThisMenuItemPos)
	RunWait, Powercfg /SetActive %powerscheme_new_guid%,, Hide UseErrorLevel

	;表示更新
	TreyIcon_Update()
	Return

;再起動
TrayMenu_Reload:
	Reload		;再起動
	Return

;終了
TrayMenu_End:
	ExitApp		;終了処理へ移行
	Return



;------------------------------------------------------------------------------
; 電源プランの一覧を作る
;------------------------------------------------------------------------------
GetPowerSchemeArrey(){
	;変数初期化
	global						;【グローバル変数モード】
	powerscheme_list	:= PS_GetList()		;電源プランリスト
	powerscheme_num		:= 0			;電源プラン数
	local guid		:= 0
	local name		:= ""
	local guidlist		:= ""
	local namelist		:= ""


	;プランリスト各行ループ
	Loop, Parse, powerscheme_list, `n
	{
		;GUIDが無い行は飛ばす
		If ( PS_FindGuid(A_LoopField) == "")
			Continue

		;GUID取得と連結
		guidlist	:= guidlist
				. PS_FindGuid(A_LoopField)
				. "`n"

		;項目名取得と連結
		namelist	:= namelist
				. PS_FindLabel(A_LoopField)
				. "`n"
	}

	;末尾の"`n"を除去
	StringTrimRight, guidlist, guidlist, 1
	StringTrimRight, namelist, namelist, 1

	;テーブル作成
	powerscheme_num :=	PowerSchemeArray_InitGuid(guidlist)
	PowerSchemeArray_InitName(namelist)

	Return
}



;------------------------------------------------------------------------------
; 現在の電源プラン情報を取得、反映
;------------------------------------------------------------------------------
;定期実行用タイマー
TreyIcon_Update_Timer:
	TreyIcon_Update(1)
	Return

;本体(polling=タイマによる定期チェック時1)
TreyIcon_Update(polling=0){
	;変数初期化
	global				;【グローバル変数モード】
	powerscheme_active_guid	:= 0	;現在の電源プランGUID
	static last_guid	:= 0	;前回値（保持）


	;アクティブプランGUID取得
	powerscheme_active_guid	:= PS_GetActiveGuid()


	;変更操作された場合のチェック
	If ( powerscheme_new_guid )
	{
		;変更失敗時はメニュー更新（削除対応）
		If ( powerscheme_active_guid != powerscheme_new_guid )
			TreyIcon_Init()

		;フラグクリア
		powerscheme_new_guid	:= 0
		last_guid		:= 0
	}


	;未知のプランの場合はメニュー更新（追加対応）
	If !( PowerSchemeArray_GetPosGuid(powerscheme_active_guid) )
		TreyIcon_Init()


	;前回値と比較して同じなら何もしない。
	If ( powerscheme_active_guid == last_guid )
		Return


	;割り込み禁止
	Critical

	;変数初期化
	local menu_name	:= ""

	;アクティブプラン名取得
	powerscheme_active_name	:= PowerSchemeArray_GetName(PowerSchemeArray_GetPosGuid(powerscheme_active_guid))

	;全チェック解除
	Loop, %powerscheme_num%
	{
		menu_name := PowerSchemeArray_GetName(A_Index)
		Menu, Tray, Uncheck, %menu_name%
	}


	Menu, Tray, Check	, %powerscheme_active_name%	;アクティブプランにチェック
	Menu, Tray, Tip		, %powerscheme_active_name%	;トレイアイコンチップ更新


	;変化検出SSTP通知
	If ( polling )
		SSTP_OnPowerSchemeUpDate(2)
	Else
		SSTP_OnPowerSchemeUpDate(0)

	;前回値保存
	last_guid := powerscheme_active_guid

	Return
}



;******************************************************************************
; SSTP 関連
;******************************************************************************
;------------------------------------------------------------------------------
; 電源プラン通知
;------------------------------------------------------------------------------
;	Sender		"PowerSchemeSelecter"
;	ShioriEVENTID	OnPowerSchemeUpDate
;	Reference0	電源プランGUID
;	Reference1	電源プラン名
;	Reference2	電源プランレベル	常に0
;	Reference3	切替理由		0:手動切替	1:自動切替(本ツールでは不出)	2:外部切替
;	Reference4	ブースト状態		常に0
;
;------------------------------------------------------------------------------
SSTP_OnPowerSchemeUpDate(reason){
	;変数初期化
	global		;【グローバル変数モード】

	local sender	:= APP_TILTE
	local eventid	:= "OnPowerSchemeUpDate"
	local ref0	:= powerscheme_active_guid
	local ref1	:= powerscheme_active_name
	local ref2	:= 0
	local ref3	:= reason
	local ref4	:= 0

	IfExist, sstpcmdR.exe
		Run, sstpcmdR.exe -s"%sender%" -i"%eventid%" -0"%ref0%" -1"%ref1%" -2"%ref2%" -3"%ref3%" -4"%ref4%",, Hide UseErrorLevel

	Return
}
